from adari_core.plots.combined import CombinedPlot
from adari_core.plots.cut import CutPlot
from adari_core.plots.panel import Panel
from adari_core.plots.histogram import HistogramPlot
from adari_core.plots.images import ImagePlot, CentralImagePlot
from adari_core.plots.points import ScatterPlot
from adari_core.plots.text import TextPlot
from adari_core.report import AdariReportBase
from adari_core.utils.utils import fetch_kw_or_default

from .hawki_utils import HawkiSetupInfo

import os

from . import HawkiReportMixin


class HawkiStdStarReport(HawkiReportMixin, AdariReportBase):
    """HAWK-I hawki_standard_process recipe report class.

    Attributes
    ----------
    - files_needed: dictionary contaning the categories needed and the file names."""

    detectors = {
        "CHIP1.INT1": [1, 1],
        "CHIP2.INT1": [2, 1],
        "CHIP3.INT1": [2, 2],
        "CHIP4.INT1": [1, 2],
    }  # dictionary values indicate typical (x,y) mosaic position
    files_needed = {
        "calib_std": "BASIC_CALIBRATED_STD",
        "matched_phot": "MATCHSTD_PHOTOM",
        "raw_im": "STD",
    }
    center_size = 200
    hist_bins_max = 50

    def __init__(self):
        super().__init__("hawki_std_star")

    def parse_sof(self):
        """
        Returns a list of files selected from a set of frames (sof).

        If more than one file fullfills the criteria, the first file
        in the array will be selected.
        """
        file_path, files_category = (
            [elem[0] for elem in self.inputs],
            [elem[1] for elem in self.inputs],
        )
        file_lists = {}
        for required_file in self.files_needed.keys():
            # Check that category matches the requirement
            if self.files_needed[required_file] in files_category:
                file_lists[required_file] = file_path[
                    files_category.index(self.files_needed[required_file])
                ]
            else:
                raise IOError(
                    "[WARNING] {} file not found".format(
                        self.files_needed[required_file]
                    )
                )
        return [file_lists]

    def generate_single_ext_panels(self, hist_clipping="sigma", hist_n_clipping=4):
        panels = {}

        exts = self.detectors.keys()
        for ext in exts:
            panel = Panel(4, 3, height_ratios=[1, 4, 4])
            calib_std = self.hdus[0]["calib_std"]
            raw_im = self.hdus[0]["raw_im"]

            # Text Plot
            vspace = 0.3
            fname = os.path.basename(str(calib_std.filename()))
            t1 = TextPlot(columns=1, v_space=vspace)
            col1 = (
                str(calib_std["PRIMARY"].header.get("INSTRUME", "Missing INSTRUME")),
                "EXTNAME: " + str(calib_std[ext].header.get("EXTNAME", "N/A")),
                "PRO CATG: "
                + str(
                    calib_std["PRIMARY"].header.get(
                        "HIERARCH ESO PRO CATG", "Missing PRO CATG"
                    )
                ),
                "FILE NAME: " + fname,
                "RAW1 NAME: "
                + str(
                    calib_std["PRIMARY"].header.get(
                        "HIERARCH ESO PRO REC1 RAW1 NAME", "Missing RAW1 NAME"
                    )
                ),
            )
            t1.add_data(col1)
            panel.assign_plot(t1, 0, 0, xext=2)

            t2 = TextPlot(columns=1, v_space=vspace)
            col2 = self.metadata
            t2.add_data(col2)
            panel.assign_plot(t2, 2, 0, xext=2)

            # Full reduced image
            vspace = 0.1
            fullfield = ImagePlot(title="Reduced field (full)")
            fullfield.add_data(calib_std[ext].data)
            panel.assign_plot(fullfield, 0, 1, xext=1, yext=1)

            # Central region of reduced image, with found standards marked
            centerfield = CentralImagePlot(
                fullfield,
                # title="Reduced field (central {} pixels)".format(self.center_size),
            )
            # Mark matched standards
            std_xvals = self.hdus[0]["matched_phot"][ext].data["X_coordinate"]
            std_yvals = self.hdus[0]["matched_phot"][ext].data["Y_coordinate"]
            scatter = ScatterPlot()
            scatter.add_data(
                (std_xvals, std_yvals),
                color="g",
                label="STD",
                marker="ring",
                markersize=5,
            )
            scatter.legend = False

            # CombinedPlot for CentralImagePlot and ScatterPlot
            combined = CombinedPlot(
                title="Reduced field (central {} pixels)".format(self.center_size)
            )
            combined.add_data(centerfield, z_order=1)
            combined.add_data(scatter, z_order=2)
            panel.assign_plot(combined, 0, 2)

            # Cut plot X
            cutX = CutPlot("x", title="Central column", x_label="y")
            cutX.y_label = fetch_kw_or_default(calib_std[ext], "BUNIT", default="ADU")
            cutX.add_data(
                fullfield,
                fullfield.get_data_coord(fullfield.data.shape[1] // 2, "x"),
                label="Full",
                color="red",
            )
            panel.assign_plot(cutX, 1, 1, xext=1, yext=1)

            # Cut plot Y
            cutY = CutPlot("y", title="Central row", x_label="x")
            cutY.y_label = fetch_kw_or_default(calib_std[ext], "BUNIT", default="ADU")
            cutY.add_data(
                fullfield,
                fullfield.get_data_coord(fullfield.data.shape[0] // 2, "y"),
                label="Full",
                color="red",
            )
            panel.assign_plot(cutY, 2, 1, xext=1, yext=1)

            # Do same for the central region
            # Cut plot X
            cutX_cent = CutPlot("x", title="Central region (Y)", x_label="y")
            cutX_cent.y_label = fetch_kw_or_default(
                calib_std[ext], "BUNIT", default="ADU"
            )
            cutX_cent.add_data(
                centerfield,
                centerfield.get_data_coord(centerfield.data.shape[1] // 2, "x"),
                label="Central",
                color="red",
            )
            panel.assign_plot(cutX_cent, 1, 2, xext=1, yext=1)

            # Cut plot Y
            cutY_cent = CutPlot("y", title="Central region (X)", x_label="x")
            cutY_cent.y_label = fetch_kw_or_default(
                calib_std[ext], "BUNIT", default="ADU"
            )
            cutY_cent.add_data(
                centerfield,
                centerfield.get_data_coord(centerfield.data.shape[0] // 2, "y"),
                label="Central",
                color="red",
            )
            panel.assign_plot(cutY_cent, 2, 2, xext=1, yext=1)

            # Plot histogram
            scaling = {}
            scaling["v_clip"] = hist_clipping
            scaling["v_clip_kwargs"] = {"nsigma": hist_n_clipping}

            histogram = HistogramPlot(
                master_data=calib_std[ext].data,
                raw_data=raw_im[ext].data if raw_im is not None else None,
                bins=self.hist_bins_max,
                title="Counts histogram",
                master_label="reduced",
                **scaling,
            )
            if raw_im is not None:
                bin_per_val = self.hist_bins_max + 1  # Dummy val to start while
                vals_per_bin = 1
                rmin, rmax = histogram.get_vlim()
                while bin_per_val > self.hist_bins_max:
                    bin_per_val = (rmax - rmin + 1) // vals_per_bin
                    vals_per_bin += 1
                histogram.bins = bin_per_val
            panel.assign_plot(histogram, 3, 1, xext=1, yext=1)

            # Metadata
            panels[panel] = {
                "report_name": "HAWK-I_std_star_ext_{}".format(ext),
                "report_description": "StandardStar_{}".format(ext),
                "report_tags": [],
            }

        return panels

    def generate_panels(self, **kwargs):
        """Create panels."""

        self.metadata = HawkiSetupInfo.std(list(self.hdus[0].values())[0])

        panels = {
            **self.generate_single_ext_panels(),
        }
        return panels


rep = HawkiStdStarReport()
